@@ -126,7 +126,8 @@ static void           window_find_next_cb            (GtkEntry        *entry,
 static void           window_findbar_close_cb        (GtkWidget       *widget,
                                                       DhWindow        *window);
 static GtkWidget *    window_new_tab_label           (DhWindow        *window,
-                                                      const gchar     *label);
+                                                      const gchar     *label,
+                                                      const GtkWidget *parent);
 static int            window_open_new_tab            (DhWindow        *window,
                                                       const gchar     *location,
                                                       gboolean         switch_focus);
@@ -137,6 +138,8 @@ static void           window_update_title            (DhWindow        *window,
 static void           window_tab_set_title           (DhWindow        *window,
                                                       WebKitWebView   *web_view,
                                                       const gchar     *title);
+static void           window_close_tab               (DhWindow *window,
+                                                      gint      page_num);
 
 G_DEFINE_TYPE (DhWindow, dh_window, GTK_TYPE_WINDOW);
 
@@ -178,16 +181,14 @@ window_activate_print (GtkAction *action,
 }
 
 static void
-window_activate_close (GtkAction *action,
-                       DhWindow  *window)
+window_close_tab (DhWindow *window,
+                  gint      page_num)
 {
         DhWindowPriv *priv;
-        gint          page_num;
         gint          pages;
 
         priv = window->priv;
 
-        page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
         gtk_notebook_remove_page (GTK_NOTEBOOK (priv->notebook), page_num);
 
         pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (priv->notebook));
@@ -200,11 +201,21 @@ window_activate_close (GtkAction *action,
         }
 }
 
+static void
+window_activate_close (GtkAction *action,
+                       DhWindow  *window)
+{
+        gint          page_num;
+
+        page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (window->priv->notebook));
+        window_close_tab (window, page_num);
+}
+
 static void
 window_activate_quit (GtkAction *action,
                       DhWindow  *window)
 {
-        gtk_main_quit ();
+        dh_base_quit (window->priv->base);
 }
 
 static void
@@ -902,6 +913,43 @@ window_populate (DhWindow *window)
         window_open_new_tab (window, NULL, TRUE);
 }
 
+
+static gchar*
+find_library_equivalent (DhWindow    *window,
+                         const gchar *uri)
+{
+        DhWindowPriv *priv;
+        gchar **components;
+        GList *iter;
+        DhLink *link;
+        gchar *book_id;
+        gchar *filename;
+        gchar *local_uri = NULL;
+
+        components = g_strsplit (uri, "/", 0);
+        book_id = components[4];
+        filename = components[6];
+
+        priv = window->priv;
+
+        for (iter = dh_base_get_keywords (priv->base); iter; iter = g_list_next (iter)) {
+                link = iter->data;
+                if (g_strcmp0 (dh_link_get_book_id (link), book_id) != 0) {
+                        continue;
+                }
+                if (g_strcmp0 (dh_link_get_file_name (link), filename) != 0) {
+                        continue;
+                }
+                local_uri = dh_link_get_uri (link);
+                break;
+        }
+
+        g_strfreev (components);
+
+        return local_uri;
+}
+
+
 static gboolean
 window_web_view_navigation_policy_decision_requested (WebKitWebView             *web_view,
                                                       WebKitWebFrame            *frame,
@@ -927,6 +975,16 @@ window_web_view_navigation_policy_decision_requested (WebKitWebView
                 return FALSE;
         }
 
+        if (strncmp (uri, "http://library.gnome.org/devel/", 31) == 0) {
+                gchar *local_uri = find_library_equivalent (window, uri);
+                if (local_uri) {
+                        webkit_web_policy_decision_ignore (policy_decision);
+                        _dh_window_display_uri (window, local_uri);
+                        g_free (local_uri);
+                        return TRUE;
+                }
+        }
+
         if (strncmp (uri, "file://", 7) != 0) {
                 webkit_web_policy_decision_ignore (policy_decision);
                 gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, NULL);
@@ -1213,7 +1271,7 @@ window_open_new_tab (DhWindow    *window,
         gtk_container_add (GTK_CONTAINER (scrolled_window), view);
         g_object_set_data (G_OBJECT (scrolled_window), "web_view", view);
 
-        label = window_new_tab_label (window, _("Empty Page"));
+        label = window_new_tab_label (window, _("Empty Page"), scrolled_window);
         gtk_widget_show_all (label);
 
         g_signal_connect (view, "title-changed",
@@ -1257,11 +1315,18 @@ static void
 close_button_clicked_cb (GtkButton *button,
                          DhWindow  *window)
 {
-        GtkAction *action_close;
+        GtkWidget *parent_tab;
+        gint       pages;
+        gint       i;
 
-        action_close = gtk_action_group_get_action (window->priv->action_group,
-                                                    "Close");
-        window_activate_close (action_close, window);
+        parent_tab = g_object_get_data (G_OBJECT (button), "parent_tab");
+        pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook));
+        for (i=0; i<pages; i++) {
+                if (gtk_notebook_get_nth_page (GTK_NOTEBOOK (window->priv->notebook), i) == parent_tab) {
+                        window_close_tab (window, i);
+                        break;
+                }
+        }
 }
 
 static void
@@ -1299,8 +1364,9 @@ tab_label_style_set_cb (GtkWidget *hbox,
  * place.
  */
 static GtkWidget*
-window_new_tab_label (DhWindow    *window,
-                      const gchar *str)
+window_new_tab_label (DhWindow        *window,
+                      const gchar     *str,
+                      const GtkWidget *parent)
 {
         GtkWidget *label;
 #ifndef GDK_WINDOWING_QUARTZ
@@ -1319,6 +1385,7 @@ window_new_tab_label (DhWindow    *window,
         gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
         gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
         gtk_widget_set_name (close_button, "devhelp-tab-close-button");
+        g_object_set_data (G_OBJECT (close_button), "parent_tab", (gpointer) parent);
 
         image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
         g_signal_connect (close_button, "clicked",
